@@ -16,16 +16,21 @@
  */
 
 #include <thrust/count.h>
+#include <thrust/functional.h>
 #include <thrust/iterator/discard_iterator.h>
 #include <thrust/iterator/retag.h>
 #include <thrust/iterator/zip_iterator.h>
 #include <thrust/partition.h>
 #include <thrust/sort.h>
 
-#include "test_real_assertions.hpp"
 #include "test_param_fixtures.hpp"
+#include "test_real_assertions.hpp"
 #include "test_utils.hpp"
 
+#if defined(THRUST_GCC_VERSION) && THRUST_GCC_VERSION >= 110000 && THRUST_GCC_VERSION < 120000
+#  define WAIVE_GCC11_FAILURES
+#endif
+
 TESTS_DEFINE(PartitionTests, FullTestsParams);
 TESTS_DEFINE(PartitionVectorTests, VectorSignedIntegerTestsParams);
 TESTS_DEFINE(PartitionIntegerTests, IntegerTestsParams);
@@ -33,7 +38,7 @@ TESTS_DEFINE(PartitionIntegerTests, IntegerTestsParams);
 template <typename T>
 struct is_even
 {
-  __host__ __device__ bool operator()(T x) const
+  THRUST_HOST_DEVICE bool operator()(T x) const
   {
     return ((int) x % 2) == 0;
   }
@@ -47,21 +52,22 @@ TYPED_TEST(PartitionVectorTests, TestPartitionSimple)
 
   SCOPED_TRACE(testing::Message() << "with device_id= " << test::set_device_from_ctest());
 
-  Vector data(5);
-  data[0] = 1;
-  data[1] = 2;
-  data[2] = 1;
-  data[3] = 1;
-  data[4] = 2;
+  // GCC 11 miscompiles and segfaults for certain versions of this test.
+  // It's not reproducible on other compilers, and the test passes when
+  // optimizations are disabled. It only affects 32-bit value types, and
+  // impacts all CPU host/device combinations tested.
+#ifdef WAIVE_GCC11_FAILURES
+  if (sizeof(T) == 4)
+  {
+    return;
+  }
+#endif
+
+  Vector data{1, 2, 1, 1, 2};
 
   Iterator iter = thrust::partition(data.begin(), data.end(), is_even<T>());
 
-  Vector ref(5);
-  ref[0] = 2;
-  ref[1] = 2;
-  ref[2] = 1;
-  ref[3] = 1;
-  ref[4] = 1;
+  Vector ref{2, 2, 1, 1, 1};
 
   ASSERT_EQ(iter - data.begin(), 2);
   ASSERT_EQ(data, ref);
@@ -75,28 +81,13 @@ TYPED_TEST(PartitionVectorTests, TestPartitionStencilSimple)
 
   SCOPED_TRACE(testing::Message() << "with device_id= " << test::set_device_from_ctest());
 
-  Vector data(5);
-  data[0] = 0;
-  data[1] = 1;
-  data[2] = 0;
-  data[3] = 0;
-  data[4] = 1;
+  Vector data{0, 1, 0, 0, 1};
 
-  Vector stencil(5);
-  stencil[0] = 1;
-  stencil[1] = 2;
-  stencil[2] = 1;
-  stencil[3] = 1;
-  stencil[4] = 2;
+  Vector stencil{1, 2, 1, 1, 2};
 
   Iterator iter = thrust::partition(data.begin(), data.end(), stencil.begin(), is_even<T>());
 
-  Vector ref(5);
-  ref[0] = 1;
-  ref[1] = 1;
-  ref[2] = 0;
-  ref[3] = 0;
-  ref[4] = 0;
+  Vector ref{1, 1, 0, 0, 0};
 
   ASSERT_EQ(iter - data.begin(), 2);
   ASSERT_EQ(data, ref);
@@ -109,12 +100,7 @@ TYPED_TEST(PartitionVectorTests, TestPartitionCopySimple)
 
   SCOPED_TRACE(testing::Message() << "with device_id= " << test::set_device_from_ctest());
 
-  Vector data(5);
-  data[0] = 1;
-  data[1] = 2;
-  data[2] = 1;
-  data[3] = 1;
-  data[4] = 2;
+  Vector data{1, 2, 1, 1, 2};
 
   Vector true_results(2);
   Vector false_results(3);
@@ -122,14 +108,9 @@ TYPED_TEST(PartitionVectorTests, TestPartitionCopySimple)
   thrust::pair<typename Vector::iterator, typename Vector::iterator> ends =
     thrust::partition_copy(data.begin(), data.end(), true_results.begin(), false_results.begin(), is_even<T>());
 
-  Vector true_ref(2);
-  true_ref[0] = 2;
-  true_ref[1] = 2;
+  Vector true_ref(2, 2);
 
-  Vector false_ref(3);
-  false_ref[0] = 1;
-  false_ref[1] = 1;
-  false_ref[2] = 1;
+  Vector false_ref(3, 1);
 
   ASSERT_EQ(2, ends.first - true_results.begin());
   ASSERT_EQ(3, ends.second - false_results.begin());
@@ -144,19 +125,9 @@ TYPED_TEST(PartitionVectorTests, TestPartitionCopyStencilSimple)
 
   SCOPED_TRACE(testing::Message() << "with device_id= " << test::set_device_from_ctest());
 
-  Vector data(5);
-  data[0] = 0;
-  data[1] = 1;
-  data[2] = 0;
-  data[3] = 0;
-  data[4] = 1;
+  Vector data{0, 1, 0, 0, 1};
 
-  Vector stencil(5);
-  stencil[0] = 1;
-  stencil[1] = 2;
-  stencil[2] = 1;
-  stencil[3] = 1;
-  stencil[4] = 2;
+  Vector stencil{1, 2, 1, 1, 2};
 
   Vector true_results(2);
   Vector false_results(3);
@@ -164,14 +135,9 @@ TYPED_TEST(PartitionVectorTests, TestPartitionCopyStencilSimple)
   thrust::pair<typename Vector::iterator, typename Vector::iterator> ends = thrust::partition_copy(
     data.begin(), data.end(), stencil.begin(), true_results.begin(), false_results.begin(), is_even<T>());
 
-  Vector true_ref(2);
-  true_ref[0] = 1;
-  true_ref[1] = 1;
+  Vector true_ref(2, 1);
 
-  Vector false_ref(3);
-  false_ref[0] = 0;
-  false_ref[1] = 0;
-  false_ref[2] = 0;
+  Vector false_ref(3, 0);
 
   ASSERT_EQ(2, ends.first - true_results.begin());
   ASSERT_EQ(3, ends.second - false_results.begin());
@@ -187,21 +153,11 @@ TYPED_TEST(PartitionVectorTests, TestStablePartitionSimple)
 
   SCOPED_TRACE(testing::Message() << "with device_id= " << test::set_device_from_ctest());
 
-  Vector data(5);
-  data[0] = 1;
-  data[1] = 2;
-  data[2] = 1;
-  data[3] = 3;
-  data[4] = 2;
+  Vector data{1, 2, 1, 3, 2};
 
   Iterator iter = thrust::stable_partition(data.begin(), data.end(), is_even<T>());
 
-  Vector ref(5);
-  ref[0] = 2;
-  ref[1] = 2;
-  ref[2] = 1;
-  ref[3] = 1;
-  ref[4] = 3;
+  Vector ref{2, 2, 1, 1, 3};
 
   ASSERT_EQ(iter - data.begin(), 2);
   ASSERT_EQ(data, ref);
@@ -210,33 +166,17 @@ TYPED_TEST(PartitionVectorTests, TestStablePartitionSimple)
 TYPED_TEST(PartitionVectorTests, TestStablePartitionStencilSimple)
 {
   using Vector   = typename TestFixture::input_type;
-  using T        = typename Vector::value_type;
   using Iterator = typename Vector::iterator;
 
   SCOPED_TRACE(testing::Message() << "with device_id= " << test::set_device_from_ctest());
 
-  Vector data(5);
-  data[0] = 1;
-  data[1] = 2;
-  data[2] = 1;
-  data[3] = 3;
-  data[4] = 2;
+  Vector data{1, 2, 1, 3, 2};
 
-  Vector stencil(5);
-  stencil[0] = 0;
-  stencil[1] = 1;
-  stencil[2] = 0;
-  stencil[3] = 0;
-  stencil[4] = 1;
+  Vector stencil{0, 1, 0, 0, 1};
 
-  Iterator iter = thrust::stable_partition(data.begin(), data.end(), stencil.begin(), thrust::identity<T>());
+  Iterator iter = thrust::stable_partition(data.begin(), data.end(), stencil.begin(), ::internal::identity{});
 
-  Vector ref(5);
-  ref[0] = 2;
-  ref[1] = 2;
-  ref[2] = 1;
-  ref[3] = 1;
-  ref[4] = 3;
+  Vector ref{2, 2, 1, 1, 3};
 
   ASSERT_EQ(iter - data.begin(), 2);
   ASSERT_EQ(data, ref);
@@ -249,12 +189,7 @@ TYPED_TEST(PartitionVectorTests, TestStablePartitionCopySimple)
 
   SCOPED_TRACE(testing::Message() << "with device_id= " << test::set_device_from_ctest());
 
-  Vector data(5);
-  data[0] = 1;
-  data[1] = 2;
-  data[2] = 1;
-  data[3] = 1;
-  data[4] = 2;
+  Vector data{1, 2, 1, 1, 2};
 
   Vector true_results(2);
   Vector false_results(3);
@@ -262,14 +197,9 @@ TYPED_TEST(PartitionVectorTests, TestStablePartitionCopySimple)
   thrust::pair<typename Vector::iterator, typename Vector::iterator> ends =
     thrust::stable_partition_copy(data.begin(), data.end(), true_results.begin(), false_results.begin(), is_even<T>());
 
-  Vector true_ref(2);
-  true_ref[0] = 2;
-  true_ref[1] = 2;
+  Vector true_ref(2, 2);
 
-  Vector false_ref(3);
-  false_ref[0] = 1;
-  false_ref[1] = 1;
-  false_ref[2] = 1;
+  Vector false_ref(3, 1);
 
   ASSERT_EQ(2, ends.first - true_results.begin());
   ASSERT_EQ(3, ends.second - false_results.begin());
@@ -280,38 +210,21 @@ TYPED_TEST(PartitionVectorTests, TestStablePartitionCopySimple)
 TYPED_TEST(PartitionVectorTests, TestStablePartitionCopyStencilSimple)
 {
   using Vector = typename TestFixture::input_type;
-  using T      = typename Vector::value_type;
 
   SCOPED_TRACE(testing::Message() << "with device_id= " << test::set_device_from_ctest());
 
-  Vector data(5);
-  data[0] = 1;
-  data[1] = 2;
-  data[2] = 1;
-  data[3] = 1;
-  data[4] = 2;
-
-  Vector stencil(5);
-  stencil[0] = false;
-  stencil[1] = true;
-  stencil[2] = false;
-  stencil[3] = false;
-  stencil[4] = true;
+  Vector data{1, 2, 1, 1, 2};
+  Vector stencil{false, true, false, false, true};
 
   Vector true_results(2);
   Vector false_results(3);
 
   thrust::pair<typename Vector::iterator, typename Vector::iterator> ends = thrust::stable_partition_copy(
-    data.begin(), data.end(), stencil.begin(), true_results.begin(), false_results.begin(), thrust::identity<T>());
+    data.begin(), data.end(), stencil.begin(), true_results.begin(), false_results.begin(), ::internal::identity{});
 
-  Vector true_ref(2);
-  true_ref[0] = 2;
-  true_ref[1] = 2;
+  Vector true_ref(2, 2);
 
-  Vector false_ref(3);
-  false_ref[0] = 1;
-  false_ref[1] = 1;
-  false_ref[2] = 1;
+  Vector false_ref(3, 1);
 
   ASSERT_EQ(2, ends.first - true_results.begin());
   ASSERT_EQ(3, ends.second - false_results.begin());
@@ -351,7 +264,7 @@ TYPED_TEST(PartitionIntegerTests, TestPartition)
       ASSERT_EQ(h_iter - h_data.begin(), d_iter - d_data.begin());
     }
   }
-};
+}
 
 TYPED_TEST(PartitionIntegerTests, TestPartitionStencil)
 {
@@ -363,6 +276,17 @@ TYPED_TEST(PartitionIntegerTests, TestPartitionStencil)
   {
     SCOPED_TRACE(testing::Message() << "with size= " << size);
 
+    // GCC 11 miscompiles and segfaults for certain versions of this test.
+    // It's not reproducible on other compilers, and the test passes when
+    // optimizations are disabled. It only affects 32-bit value types, and
+    // impacts all CPU host/device combinations tested.
+#ifdef WAIVE_GCC11_FAILURES
+    if (size == 0 && sizeof(T) == 4)
+    {
+      return;
+    }
+#endif
+
     for (auto seed : get_seeds())
     {
       SCOPED_TRACE(testing::Message() << "with seed= " << seed);
@@ -389,7 +313,7 @@ TYPED_TEST(PartitionIntegerTests, TestPartitionStencil)
       ASSERT_EQ(h_iter - h_data.begin(), d_iter - d_data.begin());
     }
   }
-};
+}
 
 TYPED_TEST(PartitionIntegerTests, TestPartitionCopy)
 {
@@ -410,8 +334,8 @@ TYPED_TEST(PartitionIntegerTests, TestPartitionCopy)
         get_random_data<T>(size, get_default_limits<T>::min(), get_default_limits<T>::max(), seed);
       thrust::device_vector<T> d_data = h_data;
 
-      size_t n_true  = thrust::count_if(h_data.begin(), h_data.end(), is_even<T>());
-      size_t n_false = size - n_true;
+      std::ptrdiff_t n_true  = thrust::count_if(h_data.begin(), h_data.end(), is_even<T>());
+      std::ptrdiff_t n_false = size - n_true;
 
       // setup output ranges
       thrust::host_vector<T> h_true_results(n_true, 0);
@@ -442,7 +366,7 @@ TYPED_TEST(PartitionIntegerTests, TestPartitionCopy)
       ASSERT_EQ(h_false_results, d_false_results);
     }
   }
-};
+}
 
 TYPED_TEST(PartitionIntegerTests, TestPartitionCopyStencil)
 {
@@ -466,8 +390,8 @@ TYPED_TEST(PartitionIntegerTests, TestPartitionCopyStencil)
       thrust::device_vector<T> d_data    = h_data;
       thrust::device_vector<T> d_stencil = h_stencil;
 
-      size_t n_true  = thrust::count_if(h_stencil.begin(), h_stencil.end(), is_even<T>());
-      size_t n_false = size - n_true;
+      std::ptrdiff_t n_true  = thrust::count_if(h_stencil.begin(), h_stencil.end(), is_even<T>());
+      std::ptrdiff_t n_false = size - n_true;
 
       // setup output ranges
       thrust::host_vector<T> h_true_results(n_true, 0);
@@ -508,7 +432,7 @@ TYPED_TEST(PartitionIntegerTests, TestPartitionCopyStencil)
       ASSERT_EQ(h_false_results, d_false_results);
     }
   }
-};
+}
 
 TYPED_TEST(PartitionIntegerTests, TestStablePartitionCopyStencil)
 {
@@ -532,8 +456,8 @@ TYPED_TEST(PartitionIntegerTests, TestStablePartitionCopyStencil)
       thrust::device_vector<T> d_data    = h_data;
       thrust::device_vector<T> d_stencil = h_stencil;
 
-      size_t n_true  = thrust::count_if(h_stencil.begin(), h_stencil.end(), is_even<T>());
-      size_t n_false = size - n_true;
+      std::ptrdiff_t n_true  = thrust::count_if(h_stencil.begin(), h_stencil.end(), is_even<T>());
+      std::ptrdiff_t n_false = size - n_true;
 
       // setup output ranges
       thrust::host_vector<T> h_true_results(n_true, 0);
@@ -574,7 +498,7 @@ TYPED_TEST(PartitionIntegerTests, TestStablePartitionCopyStencil)
       ASSERT_EQ(h_false_results, d_false_results);
     }
   }
-};
+}
 
 TYPED_TEST(PartitionIntegerTests, TestPartitionCopyToDiscardIterator)
 {
@@ -595,8 +519,8 @@ TYPED_TEST(PartitionIntegerTests, TestPartitionCopyToDiscardIterator)
         get_random_data<T>(size, get_default_limits<T>::min(), get_default_limits<T>::max(), seed);
       thrust::device_vector<T> d_data = h_data;
 
-      size_t n_true  = thrust::count_if(h_data.begin(), h_data.end(), is_even<T>());
-      size_t n_false = size - n_true;
+      std::ptrdiff_t n_true  = thrust::count_if(h_data.begin(), h_data.end(), is_even<T>());
+      std::ptrdiff_t n_false = size - n_true;
 
       // mask both ranges
       thrust::pair<thrust::discard_iterator<>, thrust::discard_iterator<>> h_result1 = thrust::partition_copy(
@@ -656,7 +580,7 @@ TYPED_TEST(PartitionIntegerTests, TestPartitionCopyToDiscardIterator)
       ASSERT_EQ_QUIET(d_reference3, d_result3);
     }
   }
-};
+}
 
 TYPED_TEST(PartitionIntegerTests, TestPartitionCopyStencilToDiscardIterator)
 {
@@ -677,12 +601,11 @@ TYPED_TEST(PartitionIntegerTests, TestPartitionCopyStencilToDiscardIterator)
         get_random_data<T>(size, get_default_limits<T>::min(), get_default_limits<T>::max(), seed);
       thrust::host_vector<T> h_stencil = get_random_data<T>(
         size, get_default_limits<T>::min(), get_default_limits<T>::max(), seed + seed_value_addition);
-
       thrust::device_vector<T> d_data    = h_data;
       thrust::device_vector<T> d_stencil = h_stencil;
 
-      size_t n_true  = thrust::count_if(h_stencil.begin(), h_stencil.end(), is_even<T>());
-      size_t n_false = size - n_true;
+      std::ptrdiff_t n_true  = thrust::count_if(h_stencil.begin(), h_stencil.end(), is_even<T>());
+      std::ptrdiff_t n_false = size - n_true;
 
       // mask both ranges
       thrust::pair<thrust::discard_iterator<>, thrust::discard_iterator<>> h_result1 = thrust::partition_copy(
@@ -772,7 +695,10 @@ TYPED_TEST(PartitionIntegerTests, TestPartitionCopyStencilToDiscardIterator)
       ASSERT_EQ_QUIET(d_reference3, d_result3);
     }
   }
-};
+}
+
+// GCC 11 miscompiles and segfaults in this tests.
+#ifndef WAIVE_GCC11_FAILURES
 
 TYPED_TEST(PartitionIntegerTests, TestStablePartition)
 {
@@ -802,7 +728,12 @@ TYPED_TEST(PartitionIntegerTests, TestStablePartition)
       ASSERT_EQ(h_iter - h_data.begin(), d_iter - d_data.begin());
     }
   }
-};
+}
+
+#endif // WAIVE_GCC11_FAILURES
+
+// GCC 11 miscompiles and segfaults in this tests.
+#ifndef WAIVE_GCC11_FAILURES
 
 TYPED_TEST(PartitionIntegerTests, TestStablePartitionStencil)
 {
@@ -835,7 +766,9 @@ TYPED_TEST(PartitionIntegerTests, TestStablePartitionStencil)
       ASSERT_EQ(h_iter - h_data.begin(), d_iter - d_data.begin());
     }
   }
-};
+}
+
+#endif // WAIVE_GCC11_FAILURES
 
 TYPED_TEST(PartitionIntegerTests, TestStablePartitionCopy)
 {
@@ -856,8 +789,8 @@ TYPED_TEST(PartitionIntegerTests, TestStablePartitionCopy)
         get_random_data<T>(size, get_default_limits<T>::min(), get_default_limits<T>::max(), seed);
       thrust::device_vector<T> d_data = h_data;
 
-      size_t n_true  = thrust::count_if(h_data.begin(), h_data.end(), is_even<T>());
-      size_t n_false = size - n_true;
+      std::ptrdiff_t n_true  = thrust::count_if(h_data.begin(), h_data.end(), is_even<T>());
+      std::ptrdiff_t n_false = size - n_true;
 
       // setup output ranges
       thrust::host_vector<T> h_true_results(n_true, 0);
@@ -884,7 +817,7 @@ TYPED_TEST(PartitionIntegerTests, TestStablePartitionCopy)
       ASSERT_EQ(h_false_results, d_false_results);
     }
   }
-};
+}
 
 TYPED_TEST(PartitionIntegerTests, TestStablePartitionCopyToDiscardIterator)
 {
@@ -905,8 +838,8 @@ TYPED_TEST(PartitionIntegerTests, TestStablePartitionCopyToDiscardIterator)
         get_random_data<T>(size, get_default_limits<T>::min(), get_default_limits<T>::max(), seed);
       thrust::device_vector<T> d_data = h_data;
 
-      size_t n_true  = thrust::count_if(h_data.begin(), h_data.end(), is_even<T>());
-      size_t n_false = size - n_true;
+      std::ptrdiff_t n_true  = thrust::count_if(h_data.begin(), h_data.end(), is_even<T>());
+      std::ptrdiff_t n_false = size - n_true;
 
       // mask both ranges
       thrust::pair<thrust::discard_iterator<>, thrust::discard_iterator<>> h_result1 = thrust::stable_partition_copy(
@@ -966,7 +899,7 @@ TYPED_TEST(PartitionIntegerTests, TestStablePartitionCopyToDiscardIterator)
       ASSERT_EQ_QUIET(d_reference3, d_result3);
     }
   }
-};
+}
 
 TYPED_TEST(PartitionIntegerTests, TestStablePartitionCopyStencilToDiscardIterator)
 {
@@ -990,8 +923,8 @@ TYPED_TEST(PartitionIntegerTests, TestStablePartitionCopyStencilToDiscardIterato
       thrust::device_vector<T> d_data    = h_data;
       thrust::device_vector<T> d_stencil = h_stencil;
 
-      size_t n_true  = thrust::count_if(h_stencil.begin(), h_stencil.end(), is_even<T>());
-      size_t n_false = size - n_true;
+      std::ptrdiff_t n_true  = thrust::count_if(h_stencil.begin(), h_stencil.end(), is_even<T>());
+      std::ptrdiff_t n_false = size - n_true;
 
       // mask both ranges
       thrust::pair<thrust::discard_iterator<>, thrust::discard_iterator<>> h_result1 = thrust::stable_partition_copy(
@@ -1081,12 +1014,12 @@ TYPED_TEST(PartitionIntegerTests, TestStablePartitionCopyStencilToDiscardIterato
       ASSERT_EQ_QUIET(d_reference3, d_result3);
     }
   }
-};
+}
 
 struct is_ordered
 {
   template <typename Tuple>
-  __host__ __device__ bool operator()(const Tuple& t) const
+  THRUST_HOST_DEVICE bool operator()(const Tuple& t) const
   {
     return thrust::get<0>(t) <= thrust::get<1>(t);
   }
@@ -1098,42 +1031,20 @@ TYPED_TEST(PartitionVectorTests, TestPartitionZipIterator)
 
   SCOPED_TRACE(testing::Message() << "with device_id= " << test::set_device_from_ctest());
 
-  Vector data1(5);
-  Vector data2(5);
-
-  data1[0] = 1;
-  data2[0] = 2;
-  data1[1] = 2;
-  data2[1] = 1;
-  data1[2] = 1;
-  data2[2] = 2;
-  data1[3] = 1;
-  data2[3] = 2;
-  data1[4] = 2;
-  data2[4] = 1;
+  Vector data1{1, 2, 1, 1, 2};
+  Vector data2{2, 1, 2, 2, 1};
 
   using Iterator      = typename Vector::iterator;
-  using IteratorTuple = typename thrust::tuple<Iterator, Iterator>;
-  using ZipIterator   = typename thrust::zip_iterator<IteratorTuple>;
+  using IteratorTuple = thrust::tuple<Iterator, Iterator>;
+  using ZipIterator   = thrust::zip_iterator<IteratorTuple>;
 
   ZipIterator begin = thrust::make_zip_iterator(thrust::make_tuple(data1.begin(), data2.begin()));
   ZipIterator end   = thrust::make_zip_iterator(thrust::make_tuple(data1.end(), data2.end()));
 
   ZipIterator iter = thrust::partition(begin, end, is_ordered());
 
-  Vector ref1(5);
-  Vector ref2(5);
-
-  ref1[0] = 1;
-  ref2[0] = 2;
-  ref1[1] = 1;
-  ref2[1] = 2;
-  ref1[2] = 1;
-  ref2[2] = 2;
-  ref1[3] = 2;
-  ref2[3] = 1;
-  ref1[4] = 2;
-  ref2[4] = 1;
+  Vector ref1{1, 1, 1, 2, 2};
+  Vector ref2{2, 2, 2, 1, 1};
 
   ASSERT_EQ(iter - begin, 3);
   ASSERT_EQ(data1, ref1);
@@ -1146,42 +1057,20 @@ TYPED_TEST(PartitionVectorTests, TestPartitionStencilZipIterator)
 
   SCOPED_TRACE(testing::Message() << "with device_id= " << test::set_device_from_ctest());
 
-  Vector data(5);
-  data[0] = 1;
-  data[1] = 0;
-  data[2] = 1;
-  data[3] = 1;
-  data[4] = 0;
+  Vector data{1, 0, 1, 1, 0};
 
-  Vector stencil1(5);
-  Vector stencil2(5);
-
-  stencil1[0] = 1;
-  stencil2[0] = 2;
-  stencil1[1] = 2;
-  stencil2[1] = 1;
-  stencil1[2] = 1;
-  stencil2[2] = 2;
-  stencil1[3] = 1;
-  stencil2[3] = 2;
-  stencil1[4] = 2;
-  stencil2[4] = 1;
+  Vector stencil1{1, 2, 1, 1, 2};
+  Vector stencil2{2, 1, 2, 2, 1};
 
   using Iterator      = typename Vector::iterator;
-  using IteratorTuple = typename thrust::tuple<Iterator, Iterator>;
-  using ZipIterator   = typename thrust::zip_iterator<IteratorTuple>;
+  using IteratorTuple = thrust::tuple<Iterator, Iterator>;
+  using ZipIterator   = thrust::zip_iterator<IteratorTuple>;
 
   ZipIterator stencil_begin = thrust::make_zip_iterator(thrust::make_tuple(stencil1.begin(), stencil2.begin()));
 
   Iterator iter = thrust::partition(data.begin(), data.end(), stencil_begin, is_ordered());
 
-  Vector ref(5);
-
-  ref[0] = 1;
-  ref[1] = 1;
-  ref[2] = 1;
-  ref[3] = 0;
-  ref[4] = 0;
+  Vector ref{1, 1, 1, 0, 0};
 
   ASSERT_EQ(iter - data.begin(), 3);
   ASSERT_EQ(data, ref);
@@ -1193,42 +1082,20 @@ TYPED_TEST(PartitionVectorTests, TestStablePartitionZipIterator)
 
   SCOPED_TRACE(testing::Message() << "with device_id= " << test::set_device_from_ctest());
 
-  Vector data1(5);
-  Vector data2(5);
-
-  data1[0] = 1;
-  data2[0] = 2;
-  data1[1] = 2;
-  data2[1] = 0;
-  data1[2] = 1;
-  data2[2] = 3;
-  data1[3] = 1;
-  data2[3] = 2;
-  data1[4] = 2;
-  data2[4] = 1;
+  Vector data1{1, 2, 1, 1, 2};
+  Vector data2{2, 0, 3, 2, 1};
 
   using Iterator      = typename Vector::iterator;
-  using IteratorTuple = typename thrust::tuple<Iterator, Iterator>;
-  using ZipIterator   = typename thrust::zip_iterator<IteratorTuple>;
+  using IteratorTuple = thrust::tuple<Iterator, Iterator>;
+  using ZipIterator   = thrust::zip_iterator<IteratorTuple>;
 
   ZipIterator begin = thrust::make_zip_iterator(thrust::make_tuple(data1.begin(), data2.begin()));
   ZipIterator end   = thrust::make_zip_iterator(thrust::make_tuple(data1.end(), data2.end()));
 
   ZipIterator iter = thrust::stable_partition(begin, end, is_ordered());
 
-  Vector ref1(5);
-  Vector ref2(5);
-
-  ref1[0] = 1;
-  ref2[0] = 2;
-  ref1[1] = 1;
-  ref2[1] = 3;
-  ref1[2] = 1;
-  ref2[2] = 2;
-  ref1[3] = 2;
-  ref2[3] = 0;
-  ref1[4] = 2;
-  ref2[4] = 1;
+  Vector ref1{1, 1, 1, 2, 2};
+  Vector ref2{2, 3, 2, 0, 1};
 
   ASSERT_EQ(data1, ref1);
   ASSERT_EQ(data2, ref2);
@@ -1241,49 +1108,27 @@ TYPED_TEST(PartitionVectorTests, TestStablePartitionStencilZipIterator)
 
   SCOPED_TRACE(testing::Message() << "with device_id= " << test::set_device_from_ctest());
 
-  Vector data(5);
-  data[0] = 1;
-  data[1] = 0;
-  data[2] = 1;
-  data[3] = 1;
-  data[4] = 0;
+  Vector data{1, 0, 1, 1, 0};
 
-  Vector stencil1(5);
-  Vector stencil2(5);
-
-  stencil1[0] = 1;
-  stencil2[0] = 2;
-  stencil1[1] = 2;
-  stencil2[1] = 0;
-  stencil1[2] = 1;
-  stencil2[2] = 3;
-  stencil1[3] = 1;
-  stencil2[3] = 2;
-  stencil1[4] = 2;
-  stencil2[4] = 1;
+  Vector stencil1{1, 2, 1, 1, 2};
+  Vector stencil2{2, 0, 3, 2, 1};
 
   using Iterator      = typename Vector::iterator;
-  using IteratorTuple = typename thrust::tuple<Iterator, Iterator>;
-  using ZipIterator   = typename thrust::zip_iterator<IteratorTuple>;
+  using IteratorTuple = thrust::tuple<Iterator, Iterator>;
+  using ZipIterator   = thrust::zip_iterator<IteratorTuple>;
 
   ZipIterator stencil_begin = thrust::make_zip_iterator(thrust::make_tuple(stencil1.begin(), stencil2.begin()));
 
   Iterator mid = thrust::stable_partition(data.begin(), data.end(), stencil_begin, is_ordered());
 
-  Vector ref(5);
-
-  ref[0] = 1;
-  ref[1] = 1;
-  ref[2] = 1;
-  ref[3] = 0;
-  ref[4] = 0;
+  Vector ref{1, 1, 1, 0, 0};
 
   ASSERT_EQ(ref, data);
   ASSERT_EQ(mid - data.begin(), 3);
 }
 
 template <typename ForwardIterator, typename Predicate>
-__host__ __device__ ForwardIterator partition(my_system& system, ForwardIterator first, ForwardIterator, Predicate)
+ForwardIterator partition(my_system& system, ForwardIterator first, ForwardIterator, Predicate)
 {
   system.validate_dispatch();
   return first;
@@ -1302,8 +1147,7 @@ TEST(PartitionTests, TestPartitionDispatchExplicit)
 }
 
 template <typename ForwardIterator, typename InputIterator, typename Predicate>
-__host__ __device__ ForwardIterator
-partition(my_system& system, ForwardIterator first, ForwardIterator, InputIterator, Predicate)
+ForwardIterator partition(my_system& system, ForwardIterator first, ForwardIterator, InputIterator, Predicate)
 {
   system.validate_dispatch();
   return first;
@@ -1322,7 +1166,7 @@ TEST(PartitionTests, TestPartitionStencilDispatchExplicit)
 }
 
 template <typename ForwardIterator, typename Predicate>
-__host__ __device__ ForwardIterator partition(my_tag, ForwardIterator first, ForwardIterator, Predicate)
+ForwardIterator partition(my_tag, ForwardIterator first, ForwardIterator, Predicate)
 {
   *first = 13;
   return first;
@@ -1340,7 +1184,7 @@ TYPED_TEST(PartitionTests, TestPartitionDispatchImplicit)
 }
 
 template <typename ForwardIterator, typename InputIterator, typename Predicate>
-__host__ __device__ ForwardIterator partition(my_tag, ForwardIterator first, ForwardIterator, InputIterator, Predicate)
+ForwardIterator partition(my_tag, ForwardIterator first, ForwardIterator, InputIterator, Predicate)
 {
   *first = 13;
   return first;
@@ -1359,7 +1203,7 @@ TYPED_TEST(PartitionTests, TestPartitionStencilDispatchImplicit)
 }
 
 template <typename InputIterator, typename OutputIterator1, typename OutputIterator2, typename Predicate>
-__host__ __device__ thrust::pair<OutputIterator1, OutputIterator2> partition_copy(
+thrust::pair<OutputIterator1, OutputIterator2> partition_copy(
   my_system& system, InputIterator, InputIterator, OutputIterator1 out_true, OutputIterator2 out_false, Predicate)
 {
   system.validate_dispatch();
@@ -1383,7 +1227,7 @@ template <typename InputIterator1,
           typename OutputIterator1,
           typename OutputIterator2,
           typename Predicate>
-__host__ __device__ thrust::pair<OutputIterator1, OutputIterator2> partition_copy(
+thrust::pair<OutputIterator1, OutputIterator2> partition_copy(
   my_system& system,
   InputIterator1,
   InputIterator1,
@@ -1409,7 +1253,7 @@ TEST(PartitionTests, TestPartitionCopyStencilDispatchExplicit)
 }
 
 template <typename InputIterator, typename OutputIterator1, typename OutputIterator2, typename Predicate>
-__host__ __device__ thrust::pair<OutputIterator1, OutputIterator2> partition_copy(
+thrust::pair<OutputIterator1, OutputIterator2> partition_copy(
   my_tag, InputIterator first, InputIterator, OutputIterator1 out_true, OutputIterator2 out_false, Predicate)
 {
   *first = 13;
@@ -1437,7 +1281,7 @@ template <typename InputIterator1,
           typename OutputIterator1,
           typename OutputIterator2,
           typename Predicate>
-__host__ __device__ thrust::pair<OutputIterator1, OutputIterator2> partition_copy(
+thrust::pair<OutputIterator1, OutputIterator2> partition_copy(
   my_tag,
   InputIterator1 first,
   InputIterator1,
@@ -1468,8 +1312,7 @@ TEST(PartitionTests, TestPartitionCopyStencilDispatchImplicit)
 }
 
 template <typename ForwardIterator, typename Predicate>
-__host__ __device__ ForwardIterator
-stable_partition(my_system& system, ForwardIterator first, ForwardIterator, Predicate)
+ForwardIterator stable_partition(my_system& system, ForwardIterator first, ForwardIterator, Predicate)
 {
   system.validate_dispatch();
   return first;
@@ -1488,8 +1331,7 @@ TEST(PartitionTests, TestStablePartitionDispatchExplicit)
 }
 
 template <typename ForwardIterator, typename InputIterator, typename Predicate>
-__host__ __device__ ForwardIterator
-stable_partition(my_system& system, ForwardIterator first, ForwardIterator, InputIterator, Predicate)
+ForwardIterator stable_partition(my_system& system, ForwardIterator first, ForwardIterator, InputIterator, Predicate)
 {
   system.validate_dispatch();
   return first;
@@ -1508,7 +1350,7 @@ TEST(PartitionTests, TestStablePartitionStencilDispatchExplicit)
 }
 
 template <typename ForwardIterator, typename Predicate>
-__host__ __device__ ForwardIterator stable_partition(my_tag, ForwardIterator first, ForwardIterator, Predicate)
+ForwardIterator stable_partition(my_tag, ForwardIterator first, ForwardIterator, Predicate)
 {
   *first = 13;
   return first;
@@ -1526,8 +1368,7 @@ TEST(PartitionTests, TestStablePartitionDispatchImplicit)
 }
 
 template <typename ForwardIterator, typename InputIterator, typename Predicate>
-__host__ __device__ ForwardIterator
-stable_partition(my_tag, ForwardIterator first, ForwardIterator, InputIterator, Predicate)
+ForwardIterator stable_partition(my_tag, ForwardIterator first, ForwardIterator, InputIterator, Predicate)
 {
   *first = 13;
   return first;
@@ -1546,7 +1387,7 @@ TEST(PartitionTests, TestStablePartitionStencilDispatchImplicit)
 }
 
 template <typename InputIterator, typename OutputIterator1, typename OutputIterator2, typename Predicate>
-__host__ __device__ thrust::pair<OutputIterator1, OutputIterator2> stable_partition_copy(
+thrust::pair<OutputIterator1, OutputIterator2> stable_partition_copy(
   my_system& system, InputIterator, InputIterator, OutputIterator1 out_true, OutputIterator2 out_false, Predicate)
 {
   system.validate_dispatch();
@@ -1570,7 +1411,7 @@ template <typename InputIterator1,
           typename OutputIterator1,
           typename OutputIterator2,
           typename Predicate>
-__host__ __device__ thrust::pair<OutputIterator1, OutputIterator2> stable_partition_copy(
+thrust::pair<OutputIterator1, OutputIterator2> stable_partition_copy(
   my_system& system,
   InputIterator1,
   InputIterator1,
@@ -1596,7 +1437,7 @@ TEST(PartitionTests, TestStablePartitionCopyStencilDispatchExplicit)
 }
 
 template <typename InputIterator, typename OutputIterator1, typename OutputIterator2, typename Predicate>
-__host__ __device__ thrust::pair<OutputIterator1, OutputIterator2> stable_partition_copy(
+thrust::pair<OutputIterator1, OutputIterator2> stable_partition_copy(
   my_tag, InputIterator first, InputIterator, OutputIterator1 out_true, OutputIterator2 out_false, Predicate)
 {
   *first = 13;
@@ -1624,7 +1465,7 @@ template <typename InputIterator1,
           typename OutputIterator1,
           typename OutputIterator2,
           typename Predicate>
-__host__ __device__ thrust::pair<OutputIterator1, OutputIterator2> stable_partition_copy(
+thrust::pair<OutputIterator1, OutputIterator2> stable_partition_copy(
   my_tag,
   InputIterator1 first,
   InputIterator1,
