@@ -15,16 +15,31 @@
  *  limitations under the License.
  */
 
+#include <thrust/functional.h>
 #include <thrust/iterator/counting_iterator.h>
 #include <thrust/iterator/permutation_iterator.h>
 #include <thrust/reduce.h>
 #include <thrust/sequence.h>
 #include <thrust/transform_reduce.h>
+#include <thrust/universal_vector.h>
 
 #include "test_param_fixtures.hpp"
 #include "test_utils.hpp"
 
+#include _THRUST_STD_INCLUDE(type_traits)
+
+using IntegralVectorTestsParams =
+  ::testing::Types<Params<thrust::host_vector<signed char>>,
+                   Params<thrust::host_vector<short>>,
+                   Params<thrust::host_vector<int>>,
+                   Params<thrust::device_vector<signed char>>,
+                   Params<thrust::device_vector<short>>,
+                   Params<thrust::device_vector<int>>,
+                   Params<thrust::universal_vector<int>>,
+                   Params<thrust::universal_host_pinned_vector<int>>>;
+
 TESTS_DEFINE(PermutationIteratorTests, FullTestsParams);
+TESTS_DEFINE(PermutationIteratorIntegralVectorTests, IntegralVectorTestsParams);
 
 TEST(PermutationIteratorTests, UsingHip)
 {
@@ -33,7 +48,7 @@ TEST(PermutationIteratorTests, UsingHip)
   ASSERT_EQ(THRUST_DEVICE_SYSTEM, THRUST_DEVICE_SYSTEM_HIP);
 }
 
-TYPED_TEST(PermutationIteratorTests, PermutationIteratorSimple)
+TYPED_TEST(PermutationIteratorTests, TestPermutationIteratorSimple)
 {
   using Vector   = typename TestFixture::input_type;
   using T        = typename Vector::value_type;
@@ -42,16 +57,11 @@ TYPED_TEST(PermutationIteratorTests, PermutationIteratorSimple)
   SCOPED_TRACE(testing::Message() << "with device_id= " << test::set_device_from_ctest());
 
   Vector source(8);
-  Vector indices(4);
+  Vector indices{3, 0, 5, 7};
 
   // initialize input
   thrust::sequence(source.begin(), source.end(), 1);
 
-  indices[0] = 3;
-  indices[1] = 0;
-  indices[2] = 5;
-  indices[3] = 7;
-
   thrust::permutation_iterator<Iterator, Iterator> begin(source.begin(), indices.begin());
   thrust::permutation_iterator<Iterator, Iterator> end(source.begin(), indices.end());
 
@@ -72,17 +82,13 @@ TYPED_TEST(PermutationIteratorTests, PermutationIteratorSimple)
   *begin = 10;
   *end   = 20;
 
-  ASSERT_EQ(source[0], 10);
-  ASSERT_EQ(source[1], 2);
-  ASSERT_EQ(source[2], 3);
-  ASSERT_EQ(source[3], 4);
-  ASSERT_EQ(source[4], 5);
-  ASSERT_EQ(source[5], 20);
-  ASSERT_EQ(source[6], 7);
-  ASSERT_EQ(source[7], 8);
+  Vector ref{10, 2, 3, 4, 5, 20, 7, 8};
+  ASSERT_EQ(source, ref);
 }
+static_assert(_THRUST_STD::is_trivially_copy_constructible<thrust::permutation_iterator<int*, int*>>::value, "");
+static_assert(_THRUST_STD::is_trivially_copyable<thrust::permutation_iterator<int*, int*>>::value, "");
 
-TYPED_TEST(PermutationIteratorTests, PermutationIteratorGather)
+TYPED_TEST(PermutationIteratorTests, TestPermutationIteratorGather)
 {
   using Vector   = typename TestFixture::input_type;
   using Iterator = typename Vector::iterator;
@@ -90,28 +96,21 @@ TYPED_TEST(PermutationIteratorTests, PermutationIteratorGather)
   SCOPED_TRACE(testing::Message() << "with device_id= " << test::set_device_from_ctest());
 
   Vector source(8);
-  Vector indices(4);
+  Vector indices{3, 0, 5, 7};
   Vector output(4, 10);
 
   // initialize input
   thrust::sequence(source.begin(), source.end(), 1);
 
-  indices[0] = 3;
-  indices[1] = 0;
-  indices[2] = 5;
-  indices[3] = 7;
-
   thrust::permutation_iterator<Iterator, Iterator> p_source(source.begin(), indices.begin());
 
   thrust::copy(p_source, p_source + 4, output.begin());
 
-  ASSERT_EQ(output[0], 4);
-  ASSERT_EQ(output[1], 1);
-  ASSERT_EQ(output[2], 6);
-  ASSERT_EQ(output[3], 8);
+  Vector ref{4, 1, 6, 8};
+  ASSERT_EQ(output, ref);
 }
 
-TYPED_TEST(PermutationIteratorTests, PermutationIteratorScatter)
+TYPED_TEST(PermutationIteratorTests, TestPermutationIteratorScatter)
 {
   using Vector   = typename TestFixture::input_type;
   using Iterator = typename Vector::iterator;
@@ -119,61 +118,43 @@ TYPED_TEST(PermutationIteratorTests, PermutationIteratorScatter)
   SCOPED_TRACE(testing::Message() << "with device_id= " << test::set_device_from_ctest());
 
   Vector source(4, 10);
-  Vector indices(4);
+  Vector indices{3, 0, 5, 7};
   Vector output(8);
 
   // initialize output
   thrust::sequence(output.begin(), output.end(), 1);
 
-  indices[0] = 3;
-  indices[1] = 0;
-  indices[2] = 5;
-  indices[3] = 7;
-
   // construct transform_iterator
   thrust::permutation_iterator<Iterator, Iterator> p_output(output.begin(), indices.begin());
 
   thrust::copy(source.begin(), source.end(), p_output);
 
-  ASSERT_EQ(output[0], 10);
-  ASSERT_EQ(output[1], 2);
-  ASSERT_EQ(output[2], 3);
-  ASSERT_EQ(output[3], 10);
-  ASSERT_EQ(output[4], 5);
-  ASSERT_EQ(output[5], 10);
-  ASSERT_EQ(output[6], 7);
-  ASSERT_EQ(output[7], 10);
+  Vector ref{10, 2, 3, 10, 5, 10, 7, 10};
+  ASSERT_EQ(output, ref);
 }
 
-TYPED_TEST(PermutationIteratorTests, MakePermutationIterator)
+TYPED_TEST(PermutationIteratorTests, TestMakePermutationIterator)
 {
   using Vector = typename TestFixture::input_type;
 
   SCOPED_TRACE(testing::Message() << "with device_id= " << test::set_device_from_ctest());
 
   Vector source(8);
-  Vector indices(4);
+  Vector indices{3, 0, 5, 7};
   Vector output(4, 10);
 
   // initialize input
   thrust::sequence(source.begin(), source.end(), 1);
 
-  indices[0] = 3;
-  indices[1] = 0;
-  indices[2] = 5;
-  indices[3] = 7;
-
   thrust::copy(thrust::make_permutation_iterator(source.begin(), indices.begin()),
                thrust::make_permutation_iterator(source.begin(), indices.begin()) + 4,
                output.begin());
 
-  ASSERT_EQ(output[0], 4);
-  ASSERT_EQ(output[1], 1);
-  ASSERT_EQ(output[2], 6);
-  ASSERT_EQ(output[3], 8);
+  Vector ref{4, 1, 6, 8};
+  ASSERT_EQ(output, ref);
 }
 
-TYPED_TEST(PermutationIteratorTests, PermutationIteratorReduce)
+TYPED_TEST(PermutationIteratorIntegralVectorTests, TestPermutationIteratorReduce)
 {
   using Vector   = typename TestFixture::input_type;
   using T        = typename Vector::value_type;
@@ -182,24 +163,19 @@ TYPED_TEST(PermutationIteratorTests, PermutationIteratorReduce)
   SCOPED_TRACE(testing::Message() << "with device_id= " << test::set_device_from_ctest());
 
   Vector source(8);
-  Vector indices(4);
+  Vector indices{3, 0, 5, 7};
   Vector output(4, 10);
 
   // initialize input
   thrust::sequence(source.begin(), source.end(), 1);
 
-  indices[0] = 3;
-  indices[1] = 0;
-  indices[2] = 5;
-  indices[3] = 7;
-
   // construct transform_iterator
   thrust::permutation_iterator<Iterator, Iterator> iter(source.begin(), indices.begin());
 
   T result1 = thrust::reduce(thrust::make_permutation_iterator(source.begin(), indices.begin()),
                              thrust::make_permutation_iterator(source.begin(), indices.begin()) + 4);
 
-  ASSERT_EQ(result1, (T) 19);
+  ASSERT_EQ(result1, 19);
 
   T result2 = thrust::transform_reduce(
     thrust::make_permutation_iterator(source.begin(), indices.begin()),
@@ -207,132 +183,106 @@ TYPED_TEST(PermutationIteratorTests, PermutationIteratorReduce)
     thrust::negate<T>(),
     T(0),
     thrust::plus<T>());
-  ASSERT_EQ(result2, (T) -19);
+  ASSERT_EQ(result2, -19);
 };
 
-TEST(PermutationIteratorTests, PermutationIteratorHostDeviceGather)
+TEST(PermutationIteratorTests, TestPermutationIteratorHostDeviceGather)
 {
   using T              = int;
-  using HostVector     = typename thrust::host_vector<T>;
-  using DeviceVector   = typename thrust::host_vector<T>;
-  using HostIterator   = typename HostVector::iterator;
-  using DeviceIterator = typename DeviceVector::iterator;
+  using HostVector     = thrust::host_vector<T>;
+  using DeviceVector   = thrust::host_vector<T>;
+  using HostIterator   = HostVector::iterator;
+  using DeviceIterator = DeviceVector::iterator;
 
   SCOPED_TRACE(testing::Message() << "with device_id= " << test::set_device_from_ctest());
 
   HostVector h_source(8);
-  HostVector h_indices(4);
+  HostVector h_indices{3, 0, 5, 7};
   HostVector h_output(4, 10);
 
   DeviceVector d_source(8);
-  DeviceVector d_indices(4);
+  DeviceVector d_indices(h_indices);
   DeviceVector d_output(4, 10);
 
   // initialize source
   thrust::sequence(h_source.begin(), h_source.end(), 1);
   thrust::sequence(d_source.begin(), d_source.end(), 1);
 
-  h_indices[0] = d_indices[0] = 3;
-  h_indices[1] = d_indices[1] = 0;
-  h_indices[2] = d_indices[2] = 5;
-  h_indices[3] = d_indices[3] = 7;
-
   thrust::permutation_iterator<HostIterator, HostIterator> p_h_source(h_source.begin(), h_indices.begin());
   thrust::permutation_iterator<DeviceIterator, DeviceIterator> p_d_source(d_source.begin(), d_indices.begin());
 
   // gather host->device
   thrust::copy(p_h_source, p_h_source + 4, d_output.begin());
 
-  ASSERT_EQ(d_output[0], 4);
-  ASSERT_EQ(d_output[1], 1);
-  ASSERT_EQ(d_output[2], 6);
-  ASSERT_EQ(d_output[3], 8);
+  DeviceVector dref{4, 1, 6, 8};
+  ASSERT_EQ(d_output, dref);
 
   // gather device->host
   thrust::copy(p_d_source, p_d_source + 4, h_output.begin());
 
-  ASSERT_EQ(h_output[0], 4);
-  ASSERT_EQ(h_output[1], 1);
-  ASSERT_EQ(h_output[2], 6);
-  ASSERT_EQ(h_output[3], 8);
+  HostVector href{4, 1, 6, 8};
+  ASSERT_EQ(h_output, href);
 }
 
-TEST(PermutationIteratorTests, PermutationIteratorHostDeviceScatter)
+TEST(PermutationIteratorTests, TestPermutationIteratorHostDeviceScatter)
 {
   using T              = int;
-  using HostVector     = typename thrust::host_vector<T>;
-  using DeviceVector   = typename thrust::host_vector<T>;
-  using HostIterator   = typename HostVector::iterator;
-  using DeviceIterator = typename DeviceVector::iterator;
+  using HostVector     = thrust::host_vector<T>;
+  using DeviceVector   = thrust::host_vector<T>;
+  using HostIterator   = HostVector::iterator;
+  using DeviceIterator = DeviceVector::iterator;
 
   SCOPED_TRACE(testing::Message() << "with device_id= " << test::set_device_from_ctest());
 
   HostVector h_source(4, 10);
-  HostVector h_indices(4);
+  HostVector h_indices{3, 0, 5, 7};
   HostVector h_output(8);
 
   DeviceVector d_source(4, 10);
-  DeviceVector d_indices(4);
+  DeviceVector d_indices(h_indices);
   DeviceVector d_output(8);
 
   // initialize source
   thrust::sequence(h_output.begin(), h_output.end(), 1);
   thrust::sequence(d_output.begin(), d_output.end(), 1);
 
-  h_indices[0] = d_indices[0] = 3;
-  h_indices[1] = d_indices[1] = 0;
-  h_indices[2] = d_indices[2] = 5;
-  h_indices[3] = d_indices[3] = 7;
-
   thrust::permutation_iterator<HostIterator, HostIterator> p_h_output(h_output.begin(), h_indices.begin());
   thrust::permutation_iterator<DeviceIterator, DeviceIterator> p_d_output(d_output.begin(), d_indices.begin());
 
   // scatter host->device
   thrust::copy(h_source.begin(), h_source.end(), p_d_output);
 
-  ASSERT_EQ(d_output[0], 10);
-  ASSERT_EQ(d_output[1], 2);
-  ASSERT_EQ(d_output[2], 3);
-  ASSERT_EQ(d_output[3], 10);
-  ASSERT_EQ(d_output[4], 5);
-  ASSERT_EQ(d_output[5], 10);
-  ASSERT_EQ(d_output[6], 7);
-  ASSERT_EQ(d_output[7], 10);
+  DeviceVector dref{10, 2, 3, 10, 5, 10, 7, 10};
+  ASSERT_EQ(d_output, dref);
 
   // scatter device->host
   thrust::copy(d_source.begin(), d_source.end(), p_h_output);
 
-  ASSERT_EQ(h_output[0], 10);
-  ASSERT_EQ(h_output[1], 2);
-  ASSERT_EQ(h_output[2], 3);
-  ASSERT_EQ(h_output[3], 10);
-  ASSERT_EQ(h_output[4], 5);
-  ASSERT_EQ(h_output[5], 10);
-  ASSERT_EQ(h_output[6], 7);
-  ASSERT_EQ(h_output[7], 10);
+  HostVector href(dref);
+  ASSERT_EQ(h_output, href);
 }
 
-TYPED_TEST(PermutationIteratorTests, PermutationIteratorWithCountingIterator)
+TYPED_TEST(PermutationIteratorTests, TestPermutationIteratorWithCountingIterator)
 {
   using Vector = typename TestFixture::input_type;
   using T      = typename Vector::value_type;
+  using diff_t = typename thrust::counting_iterator<T>::difference_type;
 
   SCOPED_TRACE(testing::Message() << "with device_id= " << test::set_device_from_ctest());
 
-  typename thrust::counting_iterator<T> input(0), index(0);
+  thrust::counting_iterator<T> input(0), index(0);
 
   // test copy()
   {
     Vector output(4, 0);
 
-    thrust::copy(thrust::make_permutation_iterator(input, index),
-                 thrust::make_permutation_iterator(input, index + output.size()),
-                 output.begin());
+    auto first = thrust::make_permutation_iterator(input, index);
+    auto last  = thrust::make_permutation_iterator(input, index + static_cast<diff_t>(output.size()));
 
-    ASSERT_EQ(output[0], 0);
-    ASSERT_EQ(output[1], 1);
-    ASSERT_EQ(output[2], 2);
-    ASSERT_EQ(output[3], 3);
+    thrust::copy(first, last, output.begin());
+
+    Vector ref{0, 1, 2, 3};
+    ASSERT_EQ(output, ref);
   }
 
   // test copy()
@@ -342,11 +292,9 @@ TYPED_TEST(PermutationIteratorTests, PermutationIteratorWithCountingIterator)
     thrust::transform(thrust::make_permutation_iterator(input, index),
                       thrust::make_permutation_iterator(input, index + 4),
                       output.begin(),
-                      thrust::identity<T>());
+                      ::internal::identity{});
 
-    ASSERT_EQ(output[0], 0);
-    ASSERT_EQ(output[1], 1);
-    ASSERT_EQ(output[2], 2);
-    ASSERT_EQ(output[3], 3);
+    Vector ref{0, 1, 2, 3};
+    ASSERT_EQ(output, ref);
   }
 }
