@@ -15,28 +15,26 @@
  *  limitations under the License.
  */
 
-#include <thrust/device_vector.h>
 #include <thrust/generate.h>
 #include <thrust/random.h>
-#include <thrust/random/detail/normal_distribution_base.h>
 
-#include <cmath>
-#include <limits>
 #include <sstream>
 
 #include "test_param_fixtures.hpp"
 #include "test_utils.hpp"
 
-#if THRUST_DEVICE_COMPILER == THRUST_DEVICE_COMPILER_HIP
+#if !_THRUST_HAS_DEVICE_SYSTEM_STD
+#  include <type_traits>
+#endif
 
 template <typename Engine>
 struct ValidateEngine
 {
-  __host__ __device__ ValidateEngine(const typename Engine::result_type value_10000)
+  THRUST_HOST_DEVICE ValidateEngine(const typename Engine::result_type value_10000)
       : m_value_10000(value_10000)
   {}
 
-  __host__ __device__ bool operator()(void) const
+  THRUST_HOST_DEVICE bool operator()(void) const
   {
     Engine e;
     e.discard(9999);
@@ -51,7 +49,7 @@ struct ValidateEngine
 template <typename Engine, bool trivial_min = (Engine::min == 0)>
 struct ValidateEngineMin
 {
-  __host__ __device__ bool operator()(void) const
+  THRUST_HOST_DEVICE bool operator()(void) const
   {
     Engine e;
 
@@ -69,7 +67,7 @@ struct ValidateEngineMin
 template <typename Engine>
 struct ValidateEngineMin<Engine, true>
 {
-  __host__ __device__ bool operator()(void) const
+  THRUST_HOST_DEVICE bool operator()(void) const
   {
     return true;
   }
@@ -78,7 +76,7 @@ struct ValidateEngineMin<Engine, true>
 template <typename Engine>
 struct ValidateEngineMax
 {
-  __host__ __device__ bool operator()(void) const
+  THRUST_HOST_DEVICE bool operator()(void) const
   {
     Engine e;
 
@@ -96,7 +94,7 @@ struct ValidateEngineMax
 template <typename Engine>
 struct ValidateEngineEqual
 {
-  __host__ __device__ bool operator()(void) const
+  THRUST_HOST_DEVICE bool operator()(void) const
   {
     bool result = true;
 
@@ -128,7 +126,7 @@ struct ValidateEngineEqual
 template <typename Engine>
 struct ValidateEngineUnequal
 {
-  __host__ __device__ bool operator()(void) const
+  THRUST_HOST_DEVICE bool operator()(void) const
   {
     bool result = true;
 
@@ -168,11 +166,11 @@ struct ValidateDistributionMin
 {
   using random_engine = Engine;
 
-  __host__ __device__ ValidateDistributionMin(const Distribution& dd)
+  THRUST_HOST_DEVICE ValidateDistributionMin(const Distribution& dd)
       : d(dd)
   {}
 
-  __host__ __device__ bool operator()(void)
+  THRUST_HOST_DEVICE bool operator()(void)
   {
     Engine e;
 
@@ -194,11 +192,11 @@ struct ValidateDistributionMax
 {
   using random_engine = Engine;
 
-  __host__ __device__ ValidateDistributionMax(const Distribution& dd)
+  THRUST_HOST_DEVICE ValidateDistributionMax(const Distribution& dd)
       : d(dd)
   {}
 
-  __host__ __device__ bool operator()(void)
+  THRUST_HOST_DEVICE bool operator()(void)
   {
     Engine e;
 
@@ -218,7 +216,7 @@ struct ValidateDistributionMax
 template <typename Distribution>
 struct ValidateDistributionEqual
 {
-  __host__ __device__ bool operator()(void) const
+  THRUST_HOST_DEVICE bool operator()(void) const
   {
     return d0 == d1;
   }
@@ -229,7 +227,7 @@ struct ValidateDistributionEqual
 template <typename Distribution>
 struct ValidateDistributionUnqual
 {
-  __host__ __device__ bool operator()(void) const
+  THRUST_HOST_DEVICE bool operator()(void) const
   {
     return d0 != d1;
   }
@@ -237,6 +235,8 @@ struct ValidateDistributionUnqual
   Distribution d0, d1;
 };
 
+#if THRUST_DEVICE_COMPILER == THRUST_DEVICE_COMPILER_HIP
+
 TEST(RandomTests, UsingHip)
 {
   SCOPED_TRACE(testing::Message() << "with device_id= " << test::set_device_from_ctest());
@@ -244,8 +244,10 @@ TEST(RandomTests, UsingHip)
   ASSERT_EQ(THRUST_DEVICE_SYSTEM, THRUST_DEVICE_SYSTEM_HIP);
 }
 
+#endif
+
 template <typename Engine, std::uint64_t value_10000>
-void TestEngineValidation(void)
+void TestEngineValidation()
 {
   // test host
   thrust::host_vector<bool> h(1);
@@ -261,7 +263,7 @@ void TestEngineValidation(void)
 }
 
 template <typename Engine>
-void TestEngineMax(void)
+void TestEngineMax()
 {
   // test host
   thrust::host_vector<bool> h(1);
@@ -277,7 +279,7 @@ void TestEngineMax(void)
 }
 
 template <typename Engine>
-void TestEngineMin(void)
+void TestEngineMin()
 {
   // test host
   thrust::host_vector<bool> h(1);
@@ -293,7 +295,7 @@ void TestEngineMin(void)
 }
 
 template <typename Engine>
-void TestEngineSaveRestore(void)
+void TestEngineSaveRestore()
 {
   // create a default engine
   Engine e0;
@@ -321,7 +323,7 @@ void TestEngineSaveRestore(void)
 }
 
 template <typename Engine>
-void TestEngineEqual(void)
+void TestEngineEqual()
 {
   ValidateEngineEqual<Engine> f;
 
@@ -339,7 +341,7 @@ void TestEngineEqual(void)
 }
 
 template <typename Engine>
-void TestEngineUnequal(void)
+void TestEngineUnequal()
 {
   ValidateEngineUnequal<Engine> f;
 
@@ -455,6 +457,16 @@ TEST(RandomTests, TestRanlux48BaseEqual)
   TestEngineEqual<Engine>();
 }
 
+#if defined(__INTEL_COMPILER) && 1800 >= __INTEL_COMPILER
+TEST(RandomTests, TestRanlux48BaseUnequal)
+{
+  SCOPED_TRACE(testing::Message() << "with device_id= " << test::set_device_from_ctest());
+
+  // ICPC has a known failure with this test.
+  // See nvbug 200414000.
+  GTEST_NONFATAL_FAILURE_("Known failure")
+}
+#else
 TEST(RandomTests, TestRanlux48BaseUnequal)
 {
   using Engine = thrust::random::ranlux48_base;
@@ -463,6 +475,7 @@ TEST(RandomTests, TestRanlux48BaseUnequal)
 
   TestEngineUnequal<Engine>();
 }
+#endif
 
 TEST(RandomTests, TestMinstdRandValidation)
 {
@@ -734,8 +747,10 @@ TEST(RandomTests, TestRanlux48Unequal)
   TestEngineUnequal<Engine>();
 }
 
+THRUST_DIAG_PUSH
+THRUST_DIAG_SUPPRESS_MSVC(4305) // truncation warning
 template <typename Distribution, typename Validator>
-void ValidateDistributionCharacteristic(void)
+void ValidateDistributionCharacteristic()
 {
   using Engine = typename Validator::random_engine;
 
@@ -755,7 +770,7 @@ void ValidateDistributionCharacteristic(void)
 
   // test distribution & engine with comparable ranges
   // only do this if they have the same result_type
-  if (thrust::detail::is_same<typename Distribution::result_type, typename Engine::result_type>::value)
+  if (_THRUST_STD::is_same<typename Distribution::result_type, typename Engine::result_type>::value)
   {
     // test Distribution with same range as engine
 
@@ -795,9 +810,10 @@ void ValidateDistributionCharacteristic(void)
 
   ASSERT_EQ(true, d[0]);
 }
+THRUST_DIAG_POP
 
 template <typename Distribution>
-void TestDistributionSaveRestore(void)
+void TestDistributionSaveRestore()
 {
   // create a default distribution
   Distribution d0(7, 13);
@@ -969,5 +985,3 @@ TEST(RandomTests, erfcinvFunction)
     }
   }
 }
-
-#endif // THRUST_DEVICE_COMPILER == THRUST_DEVICE_COMPILER_HIP
