@@ -15,14 +15,13 @@
  *  limitations under the License.
  */
 
-#include <thrust/device_vector.h>
 #include <thrust/iterator/iterator_traits.h>
 #include <thrust/iterator/retag.h>
 #include <thrust/swap.h>
 #include <thrust/system/cpp/memory.h>
 
-#include "test_real_assertions.hpp"
 #include "test_param_fixtures.hpp"
+#include "test_real_assertions.hpp"
 #include "test_utils.hpp"
 
 TESTS_DEFINE(SwapRangesTests, FullTestsParams);
@@ -42,7 +41,7 @@ ForwardIterator2 swap_ranges(my_system& system, ForwardIterator1, ForwardIterato
   return first2;
 }
 
-TEST(SwapRangesTests, SwapRangesDispatchExplicit)
+TEST(SwapRangesTests, TestSwapRangesDispatchExplicit)
 {
   SCOPED_TRACE(testing::Message() << "with device_id= " << test::set_device_from_ctest());
 
@@ -61,7 +60,7 @@ ForwardIterator2 swap_ranges(my_tag, ForwardIterator1, ForwardIterator1, Forward
   return first2;
 }
 
-TEST(SwapRangesTests, SwapRangesDispatchImplicit)
+TEST(SwapRangesTests, TestSwapRangesDispatchImplicit)
 {
   SCOPED_TRACE(testing::Message() << "with device_id= " << test::set_device_from_ctest());
 
@@ -73,51 +72,30 @@ TEST(SwapRangesTests, SwapRangesDispatchImplicit)
   ASSERT_EQ(13, vec.front());
 }
 
-TYPED_TEST(SwapRangesTests, SwapRangesSimple)
+TYPED_TEST(SwapRangesTests, TestSwapRangesSimple)
 {
   using Vector = typename TestFixture::input_type;
-  using Policy = typename TestFixture::execution_policy;
 
   SCOPED_TRACE(testing::Message() << "with device_id= " << test::set_device_from_ctest());
 
-  Vector v1(5);
-  v1[0] = 0;
-  v1[1] = 1;
-  v1[2] = 2;
-  v1[3] = 3;
-  v1[4] = 4;
+  Vector v1{0, 1, 2, 3, 4};
+  Vector v2{5, 6, 7, 8, 9};
 
-  Vector v2(5);
-  v2[0] = 5;
-  v2[1] = 6;
-  v2[2] = 7;
-  v2[3] = 8;
-  v2[4] = 9;
+  thrust::swap_ranges(v1.begin(), v1.end(), v2.begin());
 
-  thrust::swap_ranges(Policy{}, v1.begin(), v1.end(), v2.begin());
+  Vector ref1{5, 6, 7, 8, 9};
+  ASSERT_EQ(v1, ref1);
 
-  ASSERT_EQ(v1[0], 5);
-  ASSERT_EQ(v1[1], 6);
-  ASSERT_EQ(v1[2], 7);
-  ASSERT_EQ(v1[3], 8);
-  ASSERT_EQ(v1[4], 9);
-
-  ASSERT_EQ(v2[0], 0);
-  ASSERT_EQ(v2[1], 1);
-  ASSERT_EQ(v2[2], 2);
-  ASSERT_EQ(v2[3], 3);
-  ASSERT_EQ(v2[4], 4);
+  Vector ref2{0, 1, 2, 3, 4};
+  ASSERT_EQ(v2, ref2);
 }
 
-TYPED_TEST(PrimitiveSwapRangesTests, SwapRanges)
+TYPED_TEST(PrimitiveSwapRangesTests, TestSwapRanges)
 {
   using T = typename TestFixture::input_type;
 
   SCOPED_TRACE(testing::Message() << "with device_id= " << test::set_device_from_ctest());
 
-  const std::vector<size_t> sizes = get_sizes();
-  T error_margin                  = T(0.01);
-
   for (auto size : get_sizes())
   {
     SCOPED_TRACE(testing::Message() << "with size= " << size);
@@ -139,43 +117,58 @@ TYPED_TEST(PrimitiveSwapRangesTests, SwapRanges)
       thrust::swap_ranges(h1.begin(), h1.end(), h2.begin());
       thrust::swap_ranges(d1.begin(), d1.end(), d2.begin());
 
-      thrust::host_vector<T> d1_h = d1;
-      thrust::host_vector<T> d2_h = d2;
-
-      for (size_t i = 0; i < size; i++)
-      {
-        ASSERT_NEAR(h1[i], a2[i], error_margin);
-        ASSERT_NEAR(d1_h[i], a2[i], error_margin);
-        ASSERT_NEAR(h2[i], a1[i], error_margin);
-        ASSERT_NEAR(d2_h[i], a1[i], error_margin);
-      }
+      ASSERT_EQ(h1, a2);
+      ASSERT_EQ(d1, a2);
+      ASSERT_EQ(h2, a1);
+      ASSERT_EQ(d2, a1);
     }
   }
 }
 
+#if (THRUST_DEVICE_SYSTEM == THRUST_DEVICE_SYSTEM_OMP)
+TEST(SwapRangesTests, TestSwapRangesForcedIterator)
+{
+  SCOPED_TRACE(testing::Message() << "with device_id= " << test::set_device_from_ctest());
+
+  thrust::device_vector<int> A(3, 0);
+  thrust::device_vector<int> B(3, 1);
+
+  thrust::swap_ranges(thrust::retag<thrust::cpp::tag>(A.begin()),
+                      thrust::retag<thrust::cpp::tag>(A.end()),
+                      thrust::retag<thrust::cpp::tag>(B.begin()));
+
+  ASSERT_EQ(A[0], 1);
+  ASSERT_EQ(A[1], 1);
+  ASSERT_EQ(A[2], 1);
+  ASSERT_EQ(B[0], 0);
+  ASSERT_EQ(B[1], 0);
+  ASSERT_EQ(B[2], 0);
+}
+#endif
+
 struct type_with_swap
 {
-  inline __host__ __device__ type_with_swap()
+  inline THRUST_HOST_DEVICE type_with_swap()
       : m_x()
       , m_swapped(false)
   {}
 
-  inline __host__ __device__ type_with_swap(int x)
+  inline THRUST_HOST_DEVICE type_with_swap(int x)
       : m_x(x)
       , m_swapped(false)
   {}
 
-  inline __host__ __device__ type_with_swap(int x, bool s)
+  inline THRUST_HOST_DEVICE type_with_swap(int x, bool s)
       : m_x(x)
       , m_swapped(s)
   {}
 
-  inline __host__ __device__ type_with_swap(const type_with_swap& other)
+  inline THRUST_HOST_DEVICE type_with_swap(const type_with_swap& other)
       : m_x(other.m_x)
       , m_swapped(other.m_swapped)
   {}
 
-  inline __host__ __device__ bool operator==(const type_with_swap& other) const
+  inline THRUST_HOST_DEVICE bool operator==(const type_with_swap& other) const
   {
     return m_x == other.m_x && m_swapped == other.m_swapped;
   }
@@ -186,21 +179,33 @@ struct type_with_swap
   bool m_swapped;
 };
 
-inline __host__ __device__ void swap(type_with_swap& a, type_with_swap& b)
+#if !_THRUST_HAS_DEVICE_SYSTEM_STD
+namespace detail
 {
-  thrust::swap(a.m_x, b.m_x);
+THRUST_EXEC_CHECK_DISABLE
+template <typename Assignable1, typename Assignable2>
+THRUST_HOST_DEVICE inline void swap(Assignable1& a, Assignable2& b)
+{
+  Assignable1 temp = a;
+  a                = b;
+  b                = temp;
+} // end swap()
+} // namespace detail
+#endif
+
+inline THRUST_HOST_DEVICE void swap(type_with_swap& a, type_with_swap& b)
+{
+#if _THRUST_HAS_DEVICE_SYSTEM_STD
+  using _THRUST_STD::swap;
+#else
+  using ::detail::swap;
+#endif
+  swap(a.m_x, b.m_x);
   a.m_swapped = true;
   b.m_swapped = true;
 }
 
-template <class T, class H>
-void inline careful_assert(T first, H second)
-{
-  bool cond = (first == second);
-  ASSERT_EQ(cond, true);
-}
-
-TEST(SwapRangesTests, SwapRangesUserSwap)
+TEST(SwapRangesTests, TestSwapRangesUserSwap)
 {
   SCOPED_TRACE(testing::Message() << "with device_id= " << test::set_device_from_ctest());
 
@@ -213,23 +218,23 @@ TEST(SwapRangesTests, SwapRangesUserSwap)
   // check that nothing is yet swapped
   type_with_swap ref = type_with_swap(0, false);
 
-  ASSERT_EQ(ref, h_A[0]);
-  ASSERT_EQ(ref, h_A[1]);
-  ASSERT_EQ(ref, h_A[2]);
+  ASSERT_EQ_QUIET(ref, h_A[0]);
+  ASSERT_EQ_QUIET(ref, h_A[1]);
+  ASSERT_EQ_QUIET(ref, h_A[2]);
 
-  careful_assert(ref, d_A[0]);
-  careful_assert(ref, d_A[1]);
-  careful_assert(ref, d_A[2]);
+  ASSERT_EQ_QUIET(ref, d_A[0]);
+  ASSERT_EQ_QUIET(ref, d_A[1]);
+  ASSERT_EQ_QUIET(ref, d_A[2]);
 
   ref = type_with_swap(1, false);
 
-  ASSERT_EQ(ref, h_B[0]);
-  ASSERT_EQ(ref, h_B[1]);
-  ASSERT_EQ(ref, h_B[2]);
+  ASSERT_EQ_QUIET(ref, h_B[0]);
+  ASSERT_EQ_QUIET(ref, h_B[1]);
+  ASSERT_EQ_QUIET(ref, h_B[2]);
 
-  careful_assert(ref, d_B[0]);
-  careful_assert(ref, d_B[1]);
-  careful_assert(ref, d_B[2]);
+  ASSERT_EQ_QUIET(ref, d_B[0]);
+  ASSERT_EQ_QUIET(ref, d_B[1]);
+  ASSERT_EQ_QUIET(ref, d_B[2]);
 
   // swap the ranges
 
@@ -239,21 +244,21 @@ TEST(SwapRangesTests, SwapRangesUserSwap)
   // check that things were swapped
   ref = type_with_swap(1, true);
 
-  ASSERT_EQ(ref, h_A[0]);
-  ASSERT_EQ(ref, h_A[1]);
-  ASSERT_EQ(ref, h_A[2]);
+  ASSERT_EQ_QUIET(ref, h_A[0]);
+  ASSERT_EQ_QUIET(ref, h_A[1]);
+  ASSERT_EQ_QUIET(ref, h_A[2]);
 
-  careful_assert(ref, d_A[0]);
-  careful_assert(ref, d_A[1]);
-  careful_assert(ref, d_A[2]);
+  ASSERT_EQ_QUIET(ref, d_A[0]);
+  ASSERT_EQ_QUIET(ref, d_A[1]);
+  ASSERT_EQ_QUIET(ref, d_A[2]);
 
   ref = type_with_swap(0, true);
 
-  ASSERT_EQ(ref, h_B[0]);
-  ASSERT_EQ(ref, h_B[1]);
-  ASSERT_EQ(ref, h_B[2]);
+  ASSERT_EQ_QUIET(ref, h_B[0]);
+  ASSERT_EQ_QUIET(ref, h_B[1]);
+  ASSERT_EQ_QUIET(ref, h_B[2]);
 
-  careful_assert(ref, d_B[0]);
-  careful_assert(ref, d_B[1]);
-  careful_assert(ref, d_B[2]);
+  ASSERT_EQ_QUIET(ref, d_B[0]);
+  ASSERT_EQ_QUIET(ref, d_B[1]);
+  ASSERT_EQ_QUIET(ref, d_B[2]);
 }
