@@ -30,7 +30,9 @@
 #  define THRUST_DEVICE_BACKEND                 hip
 #  define THRUST_DEVICE_BACKEND_DETAIL          hip_rocprim
 #  define SPECIALIZE_DEVICE_RESOURCE_NAME(name) hip##name
-#elif THRUST_DEVICE_COMPILER == THRUST_DEVICE_COMPILER_NVCC
+#elif defined(__NVCC__) || defined(_NVHPC_CUDA)                                \
+  || (defined(__CUDA__) && THRUST_HOST_COMPILER == THRUST_HOST_COMPILER_CLANG) \
+  || THRUST_HOST_COMPILER == THRUST_HOST_COMPILER_NVRTC
 #  define THRUST_DEVICE_BACKEND                 cuda
 #  define THRUST_DEVICE_BACKEND_DETAIL          cuda_cub
 #  define SPECIALIZE_DEVICE_RESOURCE_NAME(name) cuda##name
